"""
Just some terribly written test file to compare some output to the Java Implementation
"""

import ast
import os
from collections import defaultdict
from functools import lru_cache
from codecs import open

from cas import CAS, AnnotationType
from pathlib import Path

import util

in_path = Path(os.path.abspath("test_files/in"))
out_path = Path(os.path.abspath("test_files/out"))


@lru_cache(None)
def get_name_for_annotations(annotations: frozenset, cas: CAS):
    #print("Searching: %s, %s..." % (annotations, cas))
    annos = sorted(annotations, key=lambda a: -len(util.get_covered_text(cas, a)))

    #print("...sorted...")
    for anno in annos:
        text = util.get_covered_text(cas, anno)
        if len([a for a in annos if
                util.get_covered_text(cas, a) == text]) > 1 and not util.is_pronoun(cas, anno):
            #print("...found.")
            return util.get_covered_text(cas, anno)

    #print("...default.")
    return "default"


java_output = out_path.joinpath("Fontane,-Theodor__Effi Briest_GOLD.xml.xml")
data_raw = java_output.read_text("utf8")

data_java = []

for line in data_raw.split("\n"):
    if len(line) != 0:
        data_java.append(list(ast.literal_eval(line)))
        if len(data_java[-1][1]) == 0:
            data_java[-1][1] = set()
        data_java[-1][1].discard("default")


for file in in_path.iterdir():
    if not file.name.startswith("."):
        cas = CAS(str(file))

        print("Read.")

        personToAnnotation = defaultdict(set)

        neID = "ID"

        for ne in cas.get_annotation_index(AnnotationType.NE):
            personID = ne.get(neID)

            personToAnnotation[personID].add(ne)

        personToAnnotation = {
            person: frozenset(annos) for person, annos in personToAnnotation.items()
        }

        print("Annotations for person found.")

        importantPersonNameToAnnotations = dict()
        characters = sorted(personToAnnotation.keys(), key=lambda person: -len(personToAnnotation[person]))[:10]

        annotations_to_important_person_name = defaultdict(lambda: "default")

        for character in characters:
            mentions = personToAnnotation[character]
            try:
                name = list(personToAnnotation[character])[0].get("Name") # Faster, but not universal? Jep. Works only with GOLD
            except:
                name = get_name_for_annotations(mentions, cas)
            importantPersonNameToAnnotations[name] = frozenset(mentions)
            annotations_to_important_person_name[frozenset(mentions)] = name

        print("Filtered important persons")

        print(importantPersonNameToAnnotations.keys())

        all_pos = cas.get_annotation_index(AnnotationType.POS)

        wl = list()

        final_pos = all_pos[-1].end

        for i, pos in enumerate(all_pos):
            lemma = util.get_lemma(cas, pos)

            covering_sentence = util.get_covering(cas, pos, AnnotationType.SENTENCE)
            nes_in_sentence = util.get_covered(cas, covering_sentence,
                                               AnnotationType.NE)


            important_nes_in_sentence = set()
            #[get_name_for_annotatations(tuple(personToAnnotation[ne.get("ID")]), cas) for
                                        # ne in nes_in_sentence if
                                        # get_name_for_annotatations(tuple(personToAnnotation[ne.get("ID")]),
                                        #                            cas) in importantPersonNameToAnnotations]

            for ne in nes_in_sentence:
                name = annotations_to_important_person_name[personToAnnotation[ne.get("ID")]]
                if name in importantPersonNameToAnnotations:
                    important_nes_in_sentence.add(name)
            text = util.get_covered_text(cas, pos)

            if data_java[i][1] != important_nes_in_sentence and "Roswitha." not in important_nes_in_sentence:
                print()

            wl.append((lemma, important_nes_in_sentence, text, "%i-%i" %(covering_sentence.begin, covering_sentence.end)))

            #print(pos.end)

        print("Created word tuples")

        with open(str(out_path.joinpath("py_"+file.name)), "w", "utf8") as out:
            for elm in wl:
                out.write(repr(elm))
                out.write("\n")